%macro proc_print(data=,
                  where=,
                  titlevar=);

%*
%*  Purpose: Run PROC PRINT to create multiple tables in a worksheet
%*
%*  Required: DATA, TITLEVAR
%*
%*  Usage: %PROC_PRINT(data=library.table,
%*                     where=%str(if-statement-to-subset),
%*                     titlevar=var-in-data-used-for-title);
%*
%*  See Also: Tips and Tricks for Creating Multi-Sheet Excel Workbooks
%*            the Easy Way with SAS
%*            http://support.sas.com/rnd/papers/index.html#excel2008
%*;

%*  TO DO: Add error checking, make more robust;

%local I NTABLES;

%*  Set up WHERE statement, if requested;

%if (%cmpres(&WHERE) ne ) %then %let WHERE=where &WHERE;

%*  Subset TABLE_INFO and determine the number of tables in this worksheet;

proc sql noprint;
  create table work.subset as
  select *
  from work.table_info
  &WHERE;

  select count(firstobs) into: NTABLES
  from work.subset;
quit;

%*  Run the PRINT procedure for each BY group;

%do I = 1 %to &NTABLES;

  *;
  *  Get the values used for the title, firstobs and obs
  *  from the TABLE_INFO table.
  *;

  data _null_; 

  length dsid rc 8 title $256 varfmt $32 varnum 8;

  dsid = open("&DATA", 'i');
  varnum = varnum(dsid, "&TITLEVAR");
  varfmt = varfmt(dsid, varnum);

  rc = close(dsid);

  set work.subset(firstobs=&I obs=&I);

  if (compress(varfmt) eq '')
    then title = &TITLEVAR;
    else title = putn(&TITLEVAR, varfmt);

  call symput('TITLE', title);
  call symput('FIRSTOBS', strip(put(firstobs, best.)));
  call symput('OBS', strip(put(obs, best.)));

  run;

  title "&TITLE";
  footnote;
  
  proc print data=&DATA(firstobs=&FIRSTOBS obs=&OBS) noobs label split='*';
    *  No longer need BY or PAGEBY;
    id source / style(header) = header_id;
    var val2004-val2002 / style(data)=data_comma_1;
    sum val2002-val2004 / style(total)=header_comma_1  style(grandtotal)=header_comma_1;
    label source = 'Millions of USD';
  run; quit;

%end;

%mend proc_print;



*;
*  Create a table named TABLE_INFO that contains information about the
*  tables that are to be created for the worksheet.  This information 
*  will be used by PROC PRINT to subset the original data.  The table 
*  contains values to be used for FIRSTOBS and OBS options, and in our
*  case, the value of CATEGORY will be used for a title.  We will have
*  one record for each distinct BY group value.
*;

data table_info; 

set mylib.PharmaFinancial;

by segment category;

length firstobs obs 8;

retain _all_; * General case, though not all will be meaningful;

if (first.category) then firstobs = _N_;

*  Keep 1 record per BY group with all the info we need for later;

if (last.category) then do;
  obs = _N_;
  output;
end;

run;


ods listing close;

ods tagsets.ExcelXP path="&SAMPDIR" file='PharmaFinancial.xml' style=XLsansPrinter;

  *  Titles and footnotes handled by macro code;

  ods tagsets.ExcelXP options(embedded_titles='yes'
                              auto_subtotals='yes');

*  Create a separate worksheet for each business segment;

  ods tagsets.ExcelXP options(sheet_interval='none'
                              sheet_name='Annual Geographic Segments');

  *  Call macro to create the first 3 tables;
  %PROC_PRINT(data=mylib.PharmaFinancial,
              where=segment eq 1,
              titlevar=category);

  ods tagsets.ExcelXP options(sheet_interval='none'
                              sheet_name='Annual Business Segments');

  *  Call macro to create the next 4 tables;
  %PROC_PRINT(data=mylib.PharmaFinancial,
              where=segment eq 2,
              titlevar=category);

  ods tagsets.ExcelXP options(sheet_interval='none'
                              sheet_name='Quaterly Business Segments');

  *  Call macro to create the last 3 tables;
  %PROC_PRINT(data=mylib.PharmaFinancial,
              where=segment eq 3,
              titlevar=category);

ods tagsets.ExcelXP close;